-------------------------------------------------------------------------------
-- MrCurrency
--
-- Data Broker source for Currency
--
-- Send suggestions, comments, and bugs to user@zedonline.net
-------------------------------------------------------------------------------

MRCUtils = {};

function MRCUtils:ReadCurrency()
	local result = {};
	
	-- Expand all Items
	local expanding = true;
	local expandedItems = {};
	while expanding do
		expanding = false;
		for index=1,GetCurrencyListSize() do
			name, isHeader, isExpanded, isUnused, isWatched, count, icon, version = GetCurrencyListInfo(index);
		
			if( isHeader and not isExpanded ) then
				ExpandCurrencyList(index, 1);
				tinsert(expandedItems, 1, index);
				expanding = true;
			end
		end
	end
	
	-- Process
	for index=1,GetCurrencyListSize() do
		name, isHeader, isExpanded, isUnused, isWatched, count, icon, version = GetCurrencyListInfo(index);

		if( name and not isHeader ) then
			local item = {
				name = name,
				isUnused = isUnused,
				isWatched = isWatched,
				count = count,
				icon = icon,
			};
			tinsert(result, item);
		end	
	end
	
	-- Recollapse
	for _,item in ipairs(expandedItems) do
		ExpandCurrencyList(item, 0);
	end
	
	return result;
end

function MRCUtils:MoneyString(text, size, yOffset)
	local gold, silver, copper = MRCUtils:GetMoneySplit();
	local text = text or "";

	text = MRCUtils:AppendCurrency(text, gold, "Interface\\MoneyFrame\\UI-GoldIcon", 1, size, yOffset);
	text = MRCUtils:AppendCurrency(text, silver, "Interface\\MoneyFrame\\UI-SilverIcon", 1, size, yOffset);
	text = MRCUtils:AppendCurrency(text, copper, "Interface\\MoneyFrame\\UI-CopperIcon", 1, size, yOffset);
	
	return text;	
end

function MRCUtils:AppendCurrency(text,amount,icon,displayZero,size,yOffset)
	if(amount == nil) then amount = 0 end;
	if(displayZero == nil) then displayZero = false end;
	
	size = size or 18;
	yOffset = yOffset or 0;
	
	if( amount > 0 or displayZero ) then
		if( text ~= "" ) then text = text .. " " end;
--		text = text .. amount .. " |T" .. icon .. ":0|t";
		text = text .. amount .. " |T" .. icon .. ":" .. size .. ":" .. size .. ":0:" .. yOffset .. ":64:64:4:60:4:60|t";
	end
	
	return text;
end

function MRCUtils:GetMoneySplit(money)
	local gold, silver, copper;
	
	if(money == nil) then
		money = GetMoney();
	end
	
	copper = floor(money % 100);
	money = floor(money / 100);
	silver = money % 100;
	money = floor(money / 100);
	gold = money;
	
	return gold, silver, copper;
end

function MRCUtils:TablePairPack(...)
	local t,k,v;
	t = {};
	for i = 1, select("#", ...), 2 do
		k = select(i, ...);
		v = select(i+1, ...) or 0;
		tinsert(t, k);
		tinsert(t, v);
	end
	return t;
end

function MRCUtils:TablePack(...)
    local t,v;
    t = {};
    for i = 1, select("#", ...) do
        v = select(i, ...);
		if(v ~= nil) then
        	tinsert(t, v);
		end
    end
    return t;
end

function MRCUtils:TableSize(table)
	local count = 0;

	for _, _ in pairs(table) do
		count = count + 1;
	end
	
	return count;
end

function MRCUtils:TableJoin(t1, t2)
    for k,v in pairs(t2) do
		t1[k] = v;
	end 
	return t1;  
end