-------------------------------------------------------------------------------
-- MrCurrency
--
-- Data Broker source for Currency
--
-- Send suggestions, comments, and bugs to user@zedonline.net
-------------------------------------------------------------------------------

local LibQTip = LibStub("LibQTip-1.0");
MrCurrency = LibStub("AceAddon-3.0"):NewAddon("MrCurrency",
											  "AceEvent-3.0",
											  "AceHook-3.0");

-- These are only visible to characters that have actually gotten one in the past.
-- So querying GetCurrencyInfo(402) doesn't return anything useful on a character 
-- that's never gotten a Chef's Award.
-- Shouldn't "GetCurrencyListInfo()" return these IDs as well?											
local CurrencyIds = {
	[61] = true, -- Dalaran Jewelcrafter's Token
	-- Dalaran Cooking Award (???)
	[241] = true, -- Champion's Seal
	-- Champion's Writ (???)
	[390] = true, -- Conquest Points
	[392] = true, -- Honor Points
	[395] = true, -- Justice Points
	-- Valor Points (NYI? 396?)
	[402] = true, -- Chef's Award
}		

-- Note: These can currently be gotten by querying Archaeology APIs.
-- Do we really need to keep a copy in the code for any reason?
local ArchaeologyCurrencies = {
	[384] = true, -- Dwarf
	[385] = true, -- Troll
	[393] = true, -- Fossil
	[394] = true, -- Night Elf
	[397] = true, -- Orc
	[398] = true, -- Draenei
	[399] = true, -- Vrykul
	[400] = true, -- Nerubian
	[401] = true, -- Tol'vir
}	
					
local UPDATEPERIOD = 1.0;
					
function MrCurrency:OnInitialize()
	-- Create our Data Broker Pieces
	local LDB = LibStub("LibDataBroker-1.1", true);
	if LDB then
		-- Money + Currency Display
		self.dataSource = LDB:NewDataObject("MrCurrency", {
	    	type = "data source",
			text = "Loading...",
			label = "MrCurrency",
			icon = "Interface\\Icons\\INV_Misc_Coin_02",
	    	OnClick = function(clickedframe, button)
	        	-- Do something interesting here
	    	end,
			OnEnter = function(frame)
			-- Do something else interesting here
				MrCurrency:ShowCurrencyTooltip(frame);
			end,
			OnLeave = function()
			    LibQTip:Release(MrCurrency.activeTooltip);
				MrCurrency.activeTooltip = nil;
		    end,
		});
		
		-- Archaeology Currency Display
		self.archaeologySource = LDB:NewDataObject("MrCurrency Artifacts", {
			type = "data source",
			text = "Loading...",
			label = "MrCurrency Artifacts",
			icon = "Interface\\Icons\\Trade_Archaeology",
			OnEnter = function(frame)
			-- Do something else interesting here
				MrCurrency:ShowArtifactTooltip(frame);
			end,
			OnLeave = function()
			    LibQTip:Release(MrCurrency.activeTooltip);
				MrCurrency.activeTooltip = nil;
		    end,
		})
		
		self:RegisterEvent("PLAYER_MONEY", "OnUpdate");
		self:RegisterEvent("CURRENCY_DISPLAY_UPDATE", "OnUpdate"); -- Also handled by the TokenFrame_Update hook?
		self:RegisterEvent("KNOWN_CURRENCY_TYPES_UPDATE", "OnUpdate");
		self:SecureHook("TokenFrame_LoadUI");
	end
end

function MrCurrency:TokenFrame_LoadUI()
	self:Unhook("TokenFrame_LoadUI");
	self:SecureHook("TokenFrame_Update", "OnUpdate");	
end

function MrCurrency:OnUpdate()
	self:UpdateCurrency();
	self:UpdateArtifacts();
end

function MrCurrency:UpdateCurrency()
	local text = "";

	local currencies = MRCUtils:ReadCurrency();
	
	for _, currency in ipairs(currencies) do
		if( not currency.isUnused and currency.count > 0 ) then
			text = MRCUtils:AppendCurrency(text, currency.count, currency.icon, nil, 20);
		end
	end
	
	text = MRCUtils:MoneyString(text, 12);
	
	self.dataSource.text = text;
end

function MrCurrency:ShowCurrencyTooltip(dataFrame)
	local currencies = MRCUtils:ReadCurrency();
	local maxColumns = #currencies + 2;
	local defaultAlignments = {};
	local headerAlignments = {};
	local header = {};
	local line = {};
	local playerName = UnitName("player");
	
	tinsert(defaultAlignments, "LEFT");
	tinsert(headerAlignments, "CENTER");
	tinsert(header, "");
	tinsert(line, playerName);
	
	for i=2,maxColumns-1 do
		tinsert(defaultAlignments, "RIGHT");
		tinsert(headerAlignments, "CENTER");
		tinsert(header, "|T" .. currencies[i-1].icon .. ":24:24|t");
		tinsert(line, currencies[i-1].count);
	end
	
	tinsert(defaultAlignments, "RIGHT");
	tinsert(headerAlignments, "CENTER");
	tinsert(header, "|TInterface\\Icons\\INV_Misc_Coin_02:24:24|t");
	tinsert(line, MRCUtils:MoneyString(nil, 12, 2));
	
	self.activeTooltip = LibQTip:Acquire("MrCurrencyTooltip", maxColumns, unpack(defaultAlignments));
	self.activeTooltip:SetCellMarginH(4);
	self.activeTooltip:SetCellMarginV(2);
	self.activeTooltip:AddHeader(unpack(header));
	self.activeTooltip:AddLine(unpack(line));
	
	self.activeTooltip:SmartAnchorTo(dataFrame);
	self.activeTooltip:Show();
end

function MrCurrency:UpdateArtifacts()
	local text = "";
	
	local raceCount = GetNumArchaeologyRaces();
	for index=1,raceCount do
		local _, currencyId, raceIcon, _ = GetArchaeologyRaceInfo(index);
		local currencyName, count, currencyIcon, _, _, _ = GetCurrencyInfo(currencyId);
		
		if( count and count > 0 ) then
			text = MRCUtils:AppendCurrency(text, count, "Interface\\Icons\\" .. currencyIcon, nil, 20);
		end
	end
	
	self.archaeologySource.text = text;
end

function MrCurrency:ShowArtifactTooltip(dataFrame)
	self.activeTooltip = LibQTip:Acquire("MrCurrency_ArtifactsTooltip", 2, "LEFT", "RIGHT");
	self.activeTooltip:SetCellMarginH(4);
	self.activeTooltip:SetCellMarginV(2);
	self.activeTooltip:AddHeader("Race", "Tokens");
	
	local raceCount = GetNumArchaeologyRaces();
	
	for index=1,raceCount do
		local raceName, currencyId, raceIcon, _ = GetArchaeologyRaceInfo(index);
		local currencyName, count, currencyIcon, _, _, _ = GetCurrencyInfo(currencyId);
		
		if( count and count > 0 ) then
			self.activeTooltip:AddLine(raceName, count .. " |TInterface\\Icons\\" .. currencyIcon .. ":24:24:0:4|t")
		end
		
	end
	
	self.activeTooltip:SmartAnchorTo(dataFrame);
	self.activeTooltip:Show();
end